import Foundation

enum TypeTransition: String, Codable {
    case unchanged
    case changed
    case appeared
    case disappeared
}

enum TypeDeltaReason: String, Codable {
    case normal
    case registryChanged
    case authorizationChanged
    case unsupported
    case unknown
}
