import SwiftUI

enum LegacyUIMode {
    static func isEnabled(
        forceEnabled: Bool,
        horizontalSizeClass: UserInterfaceSizeClass?,
        dynamicTypeSize: DynamicTypeSize,
        screenWidth: CGFloat
    ) -> Bool {
        forceEnabled ||
        dynamicTypeSize.isAccessibilitySize ||
        (horizontalSizeClass == .compact && screenWidth <= 390)
    }
}
